<?php
namespace SIU\SQ\Lib\procesos;

use Gelf\Logger;
use Gelf\Message;
use Gelf\Publisher;
use Gelf\Transport\UdpTransport;
use Monolog\Handler\GelfHandler;
use Phinx\Console\Command\Status;
use Phinx\Console\PhinxApplication;
use Phinx\Wrapper\TextWrapper;
use SIU\SQ\Lib\entorno\configurador;
use SIU\SQ\Lib\errores\error;
use SIU\SQ\Lib\lib;
use SIU\SQ\Lib\procesos;
use SIU\SQ\Lib\util\instanciador_afip;
use SIU\SQ\Nucleo\sq_nucleo;
use Symfony\Component\Console\Input\StringInput;
use Symfony\Component\Console\Output\NullOutput;

class proceso_validar_instalacion extends proceso
{
	protected $imprime_salida = true;
	protected $conexiones_sistemas = array();
	protected $error = array();
	protected $notificaciones = array();
	
	function __construct($conexiones_sistemas = array(), $imprime_salida = true, $entorno)
	{
		$this->conexiones_sistemas = $conexiones_sistemas;
		$this->imprime_salida = $imprime_salida;
		$this->set_entorno($entorno);
	}

	function ejecutar()
	{
		$this->ejecutar_especifico();
	}
	
	function ejecutar_especifico()
	{
		$this->iniciar("INICIANDO VALIDACIONES DE LA INSTALACION");
		
		$this->validaciones_logs();
		$this->entorno()->log()->info("INICIANDO PROCESO: Validar instalacion");
		$this->validaciones_php();
		$this->validaciones_postgres();
		$this->validaciones_version();
		$this->validaciones_integracion();
		$this->entorno()->log()->info("FINALIZANDO PROCESO: Validar instalacion");
		
		if(!empty($this->error)){
			$this->entorno()->log()->error(implode(PHP_EOL , $this->error));
			if(class_exists('\sq_toba_error')){
			    // Formatear la salida para toba
			    foreach($this->error as $key => $valor){
                    $this->error[ $key ] = str2iso($valor);
                }
                throw new \toba_error(implode("\r" , str_replace("\n", "\r", $this->error)));
			}else{
				throw new error(implode(PHP_EOL , $this->error));
			}
		}else{
			if($this->imprime_salida) echo PHP_EOL . " OK" . PHP_EOL;
		}
		if(!empty($this->notificaciones)) {
			$this->imprimir_linea();
			if ($this->imprime_salida) echo implode(PHP_EOL , $this->notificaciones);
		}

		$this->imprimir_linea();
	}
	
	function validaciones_logs()
	{
		$config = new configurador();
		try {
			$handlers = $this->entorno()->log()->getHandlers();
			foreach($handlers as $handler){
				switch(get_class($handler)){
					case "Monolog\Handler\StreamHandler" :
						// Validar permisos de escritura para los logs
						if(! is_writable(dirname($handler->getUrl()))) {
							throw new \Exception("No tiene permisos para escribir en la carpeta de logs.");
						}
						break;
					case "Monolog\Handler\GelfHandler" :
						// Validar conexion con Graylog
						$handle = fsockopen('udp://' . $config->sq_ini['graylog']['host'], $config->sq_ini['graylog']['port'], $errno, $errstr, 1);
						if (!$handle) {
							echo "$errno : $errstr <br/>";
						}
						socket_set_timeout ($handle, 1);
						$write = fwrite($handle,"x00");
						if (!$write) {
							echo "error writing to port: $index.<br/>";
							next;
						}
						$startTime = time();
						$header = fread($handle, 1);
						$endTime = time();
						$timeDiff = $endTime - $startTime;
						
						if ($timeDiff >= 1) {
							fclose($handle);
						} else {
							fclose($handle);
							throw new \Exception("No se pudo conectar con Graylog, revise la configuracion.");
						}
						break;
				}
			}
		} catch (\RuntimeException $ex){
			$this->error[] = $ex->getMessage();
			throw new \Exception($ex->getMessage());
		} catch (\Exception $ex){
			$this->error[] = $ex->getMessage();
			throw new \Exception($ex->getMessage());
		}
	}
	
	function validaciones_php()
	{
	
	}
	
	function validaciones_postgres()
	{
		// Validar encoding del motor postgres
		$encoding = $this->entorno()->db()->consultar("SHOW SERVER_ENCODING;");
		if(empty($encoding) OR trim(strtolower(str_replace('-','',$encoding[0]['server_encoding']))) !== 'utf8') {
			$this->error[] = "El encoding del motor postgres es: " . $encoding[0]['server_encoding'] . " y debería ser UTF8.";
		}
		// Validar COLLATE del motor postgres
		$collate = $this->entorno()->db()->consultar("SHOW LC_COLLATE;");
		if(empty($collate) OR trim(strtolower(str_replace('-','',$collate[0]['lc_collate']))) !== 'es_ar.utf8') {
			$this->error[] = "La variable LC_COLLATE del motor postgres es: " . $collate[0]['lc_collate'] . " y debería ser es_AR.UTF8.";
		}
		// Validar CTYPE del motor postgres
		$ctype = $this->entorno()->db()->consultar("SHOW LC_CTYPE;");
		if(empty($ctype) OR trim(strtolower(str_replace('-','',$ctype[0]['lc_ctype']))) !== 'es_ar.utf8'){
			$this->error[] = "La variable LC_CTYPE del motor postgres es: " . $ctype[0]['lc_ctype'] . " y debería ser es_AR.UTF8.";
		}
	}

	function validaciones_version()
    {
		$config = new configurador();
		$dir = $config->get_config_dir();
    	// Validar que todas las migraciones se hayan ejecutado
		$app = new PhinxApplication();
		$consoleTextWrapper = new TextWrapper($app, [
			'configuration' => $dir.'/../bin/phinx.php',
			'parser' => 'php'
		]);
		$consoleTextWrapper->getStatus('default');

		if( $consoleTextWrapper->getExitCode() ){
			$this->error[] = "La version del codigo no se corresponde con la de la base de datos." . PHP_EOL . "Por favor ejecute el siguiente comando para incorporar los ultimos cambios en la base de datos:" . PHP_EOL . " ./bin/phinx migrate";
		}
    }

	function validaciones_integracion()
	{
		// Comparar fecha local (PHP) con la fecha del servidor postgres
		$date_php = date("Y-m-d");
		$date_postgres = $this->entorno()->db()->consultar("SELECT now()::date;");
		if(empty($date_postgres) OR $date_postgres[0]['now'] !== $date_php){
			$this->error[] = "La fecha del servidor web '" . $date_php . "' y la fecha del motor postgres '" . $date_postgres[0]['now'] ."' no coinciden."; 
		}
		// Compara la hora local con la hora del servidor postgres
		$hora_php = date("H:i");
		$time_postgres = $this->entorno()->db()->consultar("SELECT to_char(now(), 'HH24:MI') as time;");
		if(empty($time_postgres) OR $time_postgres[0]['time'] !== $hora_php){
			$this->error[] = "La hora del servidor web '" . $hora_php . "' y la hora del motor postgres '" . $time_postgres[0]['time'] ."' no coinciden.";
		}
		// Validar conexiones 
		$config = new configurador();
		$dir = $config->get_config_dir();
		$id_proyecto = parse_ini_file($dir . "/../toba/proyecto.ini", TRUE)['proyecto']['id'];
		if(!isset($config->sq_ini[ $id_proyecto ]['validar_comunicacion_sistemas']) OR 
			($config->sq_ini[ $id_proyecto ]['validar_comunicacion_sistemas'] AND $config->sq_ini[ $id_proyecto ]['validar_comunicacion_sistemas'] == 'true') ) {
			// Validar restHooks
			try {
				lib::rest_hooks();
			} catch (\Exception $ex) {
				$this->error[] = "Existen errores al instanciar RestHooks. " . PHP_EOL . $ex->getMessage() . PHP_EOL;
			}
			// Validar conexion con Nucleo
			if (in_array('nucleo', $this->conexiones_sistemas)) {
				try {
					lib::entorno()->get_cliente('nucleo')->get('status');
				} catch (\Exception $ex) {
					$this->error[] = "No se pudo conectar con el Nucleo. " . PHP_EOL . $ex->getMessage() . PHP_EOL;
				}
			}
			// Validar conexion con la Unidad de venta
			if (in_array('unidad_venta', $this->conexiones_sistemas)) {
				try {
					lib::entorno()->get_cliente('unidad_venta')->get('status');
				} catch (\Exception $ex) {
					$this->error[] = "No se pudo conectar con la Unidad de venta. " . PHP_EOL . $ex->getMessage() . PHP_EOL;
				}
			}
			// Validar conexion con SQ-Pagos-Backend
			if (in_array('sq_pagos_backend', $this->conexiones_sistemas)) {
				try {
					lib::entorno()->get_cliente('sq_pagos_backend')->get('status');
				} catch (\Exception $ex) {
					$this->error[] = "No se pudo conectar con SQ-Pagos-Backend. " . PHP_EOL . $ex->getMessage() . PHP_EOL;
				}
			}
			// Validar conexion con Pilaga
			if (in_array('pilaga', $this->conexiones_sistemas)) {
				try {
					lib::entorno()->get_cliente('pilaga')->get('cuentas-tesoreria');
				} catch (\Exception $ex) {
					$this->error[] = "No se pudo conectar con SIU-Pilaga. " . PHP_EOL . $ex->getMessage() . PHP_EOL;
				}
			}
			// Validar conexion con Guarani
			if (in_array('guarani', $this->conexiones_sistemas)) {
				try {
					lib::entorno()->get_cliente('guarani')->get('cursos');
				} catch (\Exception $ex) {
					$this->error[] = "No se pudo conectar con SIU-Guarani. " . PHP_EOL . $ex->getMessage() . PHP_EOL;
				}
			}
			// Validar configuracion basica de AFIP
            if (in_array('afip', $this->conexiones_sistemas)) {
                try {
                    if($config->sq_ini['factura_afip']['cuit'] !== '00000000000') {
                        $intanciador_afip = new instanciador_afip($this->entorno());
                    }
                } catch (\Exception $ex) {
                    $this->error[] = "No se pudo conectar con AFIP. " . PHP_EOL . $ex->getMessage() . PHP_EOL;
                }
            }
		} else {
			$this->notificaciones[] = "No se estan validando las conexiones con los sistemas" . PHP_EOL . "Elimine la entrada 'validar_comunicacion_sistemas' en el archivo 'sq.ini' para activarla";
		}
	}

	function iniciar($msg)
	{
		$params = $this->entorno()->db()->get_parametros();
		if($this->imprime_salida) echo PHP_EOL . " ------------------------------------------------------------------------";
		if($this->imprime_salida) echo PHP_EOL . "  " . $msg . " del proyecto";
		if($this->imprime_salida) echo PHP_EOL . " ------------------------------------------------------------------------";
	}
	
	function imprimir_linea()
	{
		if($this->imprime_salida) echo PHP_EOL . " ------------------------------------------------------------------------" . PHP_EOL;
	}
}
