<?php
namespace SIU\SQ\Lib\util;

use SIU\SQ\Lib\entorno\configurador;
use SIU\SQ\Lib\errores\error;
use SIU\SQ\Lib\errores\error_negocio;

class instanciador_afip
{
    protected $entorno;
    protected $afip;
    
    public function __construct($entorno)
    {
        $this->entorno = $entorno;
        
        $this->cargar_configuraciones();

        $this->validar();
    }
    
    public function getAfip()
    {
        return $this->afip; 
    }

    //---- CARGAR CONFIGURACIONES ---------------

    protected function cargar_configuraciones()
    {
        $configurador = new configurador();
        $credenciales = $this->get_credenciales();

        $config = [
            'CUIT' => $configurador->sq_ini['factura_afip']['cuit'],
            'production' => ($configurador->sq_ini['factura_afip']['produccion'] == "true") ? true : false,
            'cert' => $credenciales['cert'],
            'key' => $credenciales['private_key'],
            'token_dir' => $credenciales['token_dir']
        ];

        try {
            $this->afip = new \SIU\Afip\Afip($config);
            $this->entorno->log()->info("Se cargaron las configuraciones.");
        } catch (\Exception $e) {
            $msg = "Se encontraron errores al cargar las configuraciones AFIP.";
            throw new \Exception($msg . " " . $e);
        }

        if($configurador->sq_ini['factura_afip']['produccion'] == "true"){
            $this->entorno->log()->info("Se encuentra en MODO PRODUCCION.");
        }else {
            $this->entorno->log()->info("Se encuentra en MODO TEST.");
        }
    }

    protected function get_credenciales()
    {
        $config = new configurador();
        $path = realpath($config->get_config_dir()) . "/credenciales_afip";
        $datos['cert'] = $path . "/certificado.crt";
        $datos['private_key'] = $path . "/clave_privada";
        $datos['token_dir'] = $path . "/";

        return $datos;
    }

    //---- VALIDACIONES -------------


    protected function validar()
    {
        // Si se encuentra configurado un nuevo de cuit...
        $config = new configurador();
        if($config->sq_ini['factura_afip']['cuit'] !== '00000000000') {
            $this->validar_formato_numero_cuit();

            $this->validar_clave_privada_y_certificado();

            $this->validar_estado_servidor_afip();

            $this->validar_numero_cuit_en_afip();
        }else{
            throw new \Exception("No se encuentra configurado el número de CUIT.");
        }
    }

    protected function validar_formato_numero_cuit()
    {
        // Validar que el numero de CUIT solo contenga numeros
        $config = new configurador();
        $numeros = preg_replace('/[0-9]/', '', $config->sq_ini['factura_afip']['cuit']);
        if(strlen($numeros) > 0){
            throw new \Exception("El número de CUIT '" . $config->sq_ini['factura_afip']['cuit'] . "' no es válido, solo debe contener números.");
        }
        if(strlen($config->sq_ini['factura_afip']['cuit']) !== 11){
            throw new \Exception("El número '" . $config->sq_ini['factura_afip']['cuit'] . "' no es válido, debe contener 11 dígitos.");
        }
    }

    protected function validar_clave_privada_y_certificado()
    {
        // Validar que los certificados esten generados correctamente
        if( !$this->afip->getServiceTA('wsfe')){
            throw new \Exception("Existen errores con los certificados AFIP." );
        }
    }

    protected function validar_estado_servidor_afip()
    {
        $factura_electronica = new \SIU\Afip\WebService\FacturaElectronica($this->afip);
        $result = $factura_electronica->getEstadoServicio();
        if($result->AppServer !== 'OK' OR $result->DbServer !== 'OK' OR $result->AuthServer !== 'OK'){
            $msg = "Se encontraron errores en el estado del servidor AFIP. "
                . PHP_EOL . " AppServer: " . $result->AppServer
                . PHP_EOL . " DbServer: " . $result->DbServer
                . PHP_EOL . " AuthServer: " . $result->AuthServer;
            throw new \Exception($msg);
        }
        $this->entorno->log()->info("Estado del servidor AFIP: OK.");
    }

    protected function validar_numero_cuit_en_afip()
    {
        // No es la mejor validacion pero se valida el numero de cuit en AFIP
        $factura_electronica = new \SIU\Afip\WebService\FacturaElectronica($this->afip);
        $factura_electronica->getTiposCbte();
    }
}